var __create = Object.create;
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  HyperModel: () => HyperModel,
  HyperModelError: () => HyperModelError,
  ModelAssertionError: () => ModelAssertionError,
  OrderItem: () => import_sequelize2.OrderItem,
  Schema: () => Schema,
  buildModels: () => buildModels,
  converge: () => converge,
  dropSchema: () => dropSchema,
  generateHexToken: () => generateHexToken,
  hashPassword: () => hashPassword,
  migrate: () => migrate,
  syncSchema: () => syncSchema,
  tableBuilder: () => tableBuilder,
  truncate: () => truncate,
  verifyPassword: () => verifyPassword
});
module.exports = __toCommonJS(src_exports);
var st = __toESM(require("sequelize-typescript"));

// src/hypermodel.ts
var import_sequelize_typescript = require("sequelize-typescript");
var HyperModel = class extends import_sequelize_typescript.Model {
  static async exists(options) {
    const c = await this.count(options);
    return c !== 0;
  }
};
var HyperModelError = class extends Error {
  constructor(message) {
    super(message);
    this.isModelError = true;
    Object.setPrototypeOf(this, new.target.prototype);
  }
};
var ModelAssertionError = class extends Error {
  constructor(message) {
    super(message);
    this.isAssertionError = true;
    Object.setPrototypeOf(this, new.target.prototype);
  }
};
function UUIDPK(t, prop) {
  (0, import_sequelize_typescript.Column)(t, prop);
  (0, import_sequelize_typescript.PrimaryKey)(t, prop);
  (0, import_sequelize_typescript.IsUUID)(4)(t, prop);
  (0, import_sequelize_typescript.AllowNull)(false)(t, prop);
  (0, import_sequelize_typescript.Default)(import_sequelize_typescript.DataType.UUIDV4)(t, prop);
}

// src/utils.ts
var import_crypto = require("crypto");
var import_util = require("util");
var import_assert = require("assert");
function generateHexToken(size) {
  return (0, import_crypto.randomBytes)(size).toString("hex");
}
async function parsePassword(password) {
  const salt = (await (0, import_util.promisify)(import_crypto.randomBytes)(16)).toString("hex");
  const iterations = 1e5;
  const keylen = 64;
  const digest = "sha256";
  const derivedKey = await (0, import_util.promisify)(import_crypto.pbkdf2)(password, salt, iterations, keylen, digest);
  return `pbkdf2_${digest}$${iterations}$${salt}$${derivedKey.toString("hex")}`;
}
async function hashPassword(plainTextPassword, options = {}) {
  if (plainTextPassword.trim().length < 6) {
    throw new Error("Password invalid: too short");
  }
  if (options.legacy) {
    return parsePassword(plainTextPassword);
  }
  const saltBuffer = await (0, import_util.promisify)(import_crypto.randomBytes)(16);
  const iterations = 15e4;
  const keylen = 32;
  const digest = "sha256";
  const derivedKeyBuffer = await (0, import_util.promisify)(import_crypto.pbkdf2)(plainTextPassword, saltBuffer, iterations, keylen, digest);
  const salt = saltBuffer.toString("base64");
  const derivedKey = derivedKeyBuffer.toString("base64");
  return `pbkdf2_${digest}$${iterations}$${salt}$${derivedKey}`;
}
async function verifyPassword(plainTextPassword, passwordHash, options = {}) {
  const legacy = options.legacy || false;
  const [algorithm, iterations, salt, derivedKey] = passwordHash.split("$");
  (0, import_assert.strictEqual)(algorithm, "pbkdf2_sha256", "Invalid algorithm.");
  (0, import_assert.strictEqual)(typeof iterations, "string", "Invalid password format.");
  (0, import_assert.strictEqual)(typeof salt, "string", "Invalid password format.");
  (0, import_assert.strictEqual)(typeof derivedKey, "string", "Invalid password format.");
  (0, import_assert.strictEqual)(isNaN(parseInt(iterations, 10)), false, "Invalid password format.");
  const saltBuffer = Buffer.from(salt, legacy ? "hex" : "base64");
  const derivedKeyBuffer = Buffer.from(derivedKey, legacy ? "hex" : "base64");
  const digest = "sha256";
  const password = await (0, import_util.promisify)(import_crypto.pbkdf2)(plainTextPassword, legacy ? saltBuffer.toString("hex") : saltBuffer, parseInt(iterations, 10), derivedKeyBuffer.length, digest);
  return (0, import_crypto.timingSafeEqual)(password, derivedKeyBuffer);
}

// src/models.ts
var import_glob = __toESM(require("glob"));
var import_sequelize_typescript2 = require("sequelize-typescript");
var import_lodash = __toESM(require("lodash"));
var load = (patt) => import_glob.default.sync(patt).map((f) => {
  let mod = null;
  try {
    mod = require(f);
  } catch (e) {
    console.log(`error dynamically loading '${f}'
${e.toString()}`);
    throw e;
  }
  return mod.default || mod;
});
var isTrue = (val) => val === true || val === "true";
var POOL_MAX_DEFAULT = 5;
var POOL_MIN_DEFAULT = 1;
var POOL_ACQUIRE_DEFAULT = 3e4;
var POOL_IDLE_DEFAULT = 1e4;
var createSequelizeConfig = async ({
  logger,
  config,
  modelsRoot
}) => {
  var _a, _b, _c, _d;
  const models = import_lodash.default.reduce(load(modelsRoot), import_lodash.default.merge);
  const max_pool_connection = ((_a = config.pool) == null ? void 0 : _a.max) || POOL_MAX_DEFAULT;
  let min_pool_connection = ((_b = config.pool) == null ? void 0 : _b.min) || POOL_MIN_DEFAULT;
  if (min_pool_connection > max_pool_connection) {
    logger.error(`max_pool_connection: ${max_pool_connection} is smaller than min_pool_connection: ${min_pool_connection}. setting min=max`);
    min_pool_connection = max_pool_connection;
  }
  const uri = config.uri;
  return {
    uri,
    models,
    sequelizeConfig: {
      logging: config.logging,
      benchmark: isTrue(config.logging),
      native: config.native,
      ssl: config.ssl,
      models: import_lodash.default.values(models),
      pool: {
        max: max_pool_connection,
        min: min_pool_connection,
        acquire: ((_c = config.pool) == null ? void 0 : _c.acquire) || POOL_ACQUIRE_DEFAULT,
        idle: ((_d = config.pool) == null ? void 0 : _d.idle) || POOL_IDLE_DEFAULT
      }
    }
  };
};
var buildModels = async ({
  logger,
  config,
  modelsRoot
}) => {
  const { uri, sequelizeConfig, models } = await createSequelizeConfig({
    logger,
    config,
    modelsRoot
  });
  const connection = new import_sequelize_typescript2.Sequelize(uri, sequelizeConfig);
  return { models, connection };
};

// src/migrate.ts
var import_sequelize = require("sequelize");
var import_lodash2 = __toESM(require("lodash"));
var import_debug = __toESM(require("debug"));
var debug = (0, import_debug.default)("hypermodel:migrate");
var tableBuilder = {
  build: (...args) => import_lodash2.default.reduce(args, import_lodash2.default.merge, {}),
  uniqueConstraint: (name, fields, opts) => __spreadValues({
    fields,
    type: "unique",
    name
  }, opts),
  intFK: (name, toTable, toKey, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.INTEGER,
      allowNull: false,
      references: {
        model: toTable,
        key: toKey
      }
    }, opts)
  }),
  intPK: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.INTEGER,
      allowNull: false,
      autoIncrement: true,
      primaryKey: true
    }, opts)
  }),
  json: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.JSON
    }, opts)
  }),
  text: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.TEXT
    }, opts)
  }),
  string: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.STRING
    }, opts)
  }),
  nonNullString: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.STRING,
      allowNull: false
    }, opts)
  }),
  uniqueString: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.STRING,
      allowNull: false,
      unique: true
    }, opts)
  }),
  date: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.DATE
    }, opts)
  }),
  nonNullDate: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.DATE,
      allowNull: false
    }, opts)
  }),
  bool: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.BOOLEAN
    }, opts)
  }),
  int: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.INTEGER
    }, opts)
  }),
  nonNullInt: (name, opts) => ({
    [name]: __spreadValues({
      type: import_sequelize.INTEGER,
      allowNull: false
    }, opts)
  }),
  timestamps: (opts) => ({
    createdAt: __spreadValues({
      type: import_sequelize.DATE,
      allowNull: false
    }, opts),
    updatedAt: __spreadValues({
      type: import_sequelize.DATE,
      allowNull: false
    }, opts)
  })
};
var logTransform = ({
  event,
  name,
  durationSeconds
}) => {
  if (event === "migrating") {
    return `===>    up   | ${name}`;
  } else if (event === "migrated") {
    return `             | ${name} (in ${durationSeconds}s)`;
  } else if (event === "reverting") {
    return `===>    down | ${name}`;
  } else if (event === "reverted") {
    return `             | ${name} (in ${durationSeconds}s)`;
  } else {
    return `${event}: ${name} (tbd)`;
  }
};
var migrate = (cmd, _id) => async ({
  migrationsRoot,
  logger,
  connection
}) => {
  const { Umzug, SequelizeStorage } = require("umzug");
  const migrationsPath = migrationsRoot;
  debug("migration root is:", migrationsPath);
  const storage = new SequelizeStorage({
    sequelize: connection
  });
  const umzug = new Umzug({
    storage,
    logger: __spreadProps(__spreadValues({}, logger), { info: (msg) => logger.info(logTransform(msg)) }),
    migrations: {
      glob: migrationsPath,
      resolve: ({ name, path, context }) => {
        const migration = require(path);
        const newContext = __spreadProps(__spreadValues({}, context), {
          query: connection.getQueryInterface(),
          t: tableBuilder
        });
        return {
          name,
          up: async () => await migration.up(newContext),
          down: async () => await migration.down(newContext)
        };
      }
    }
  });
  const { yellow, bold } = logger.colors;
  const pref = bold(yellow("models:"));
  logger.info(`${pref} === migrations: start ===`);
  await umzug.up();
  logger.info(`${pref} === migrations: done  ===`);
  if (cmd !== "auto") {
    await connection.close();
  }
};

// src/converge.ts
var import_debug2 = __toESM(require("debug"));
var debug2 = (0, import_debug2.default)("hypermodel:converge");
var dropSchema = async ({
  logger,
  connection
}) => {
  const { yellow, bold } = logger.colors;
  logger.info(`${bold(yellow("models:"))} dropping schema`);
  await connection.query(`DROP TABLE IF EXISTS "SequelizeMeta";`);
  await connection.drop();
};
var syncSchema = async ({
  logger,
  connection
}) => {
  const { yellow, bold } = logger.colors;
  logger.info(`${bold(yellow("models:"))} syncing schema`);
  await connection.sync({ force: false });
};
var truncate = async ({
  logger,
  connection,
  models
}) => {
  const { yellow, bold } = logger.colors;
  logger.info(`${bold(yellow("models:"))} truncating tables`);
  for (const model of Object.values(models)) {
    const m = model;
    await m.truncate({ restartIdentity: true, cascade: true, force: true });
    if (connection.getDialect() === "sqlite") {
      await connection.query("PRAGMA foreign_keys = OFF;");
      await connection.query(`DELETE FROM ${m.tableName};`);
      await connection.query(`DELETE FROM sqlite_sequence;`);
    }
  }
};
var converge = async ({
  migrationsRoot,
  models,
  config,
  connection,
  logger
}) => {
  debug2("configuration %o", config);
  if (config.dropSchema) {
    await dropSchema({ logger, connection });
  }
  if (config.synchronize) {
    await syncSchema({ logger, connection });
  }
  if (config.migrate) {
    const mig = migrate("auto");
    await mig({ migrationsRoot, connection, logger });
  }
  if (config.truncate) {
    await truncate({ logger, connection, models });
  }
};

// src/types.ts
var import_sequelize2 = require("sequelize");

// src/index.ts
var Schema = __spreadProps(__spreadValues({}, st), { UUIDPK });
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  HyperModel,
  HyperModelError,
  ModelAssertionError,
  OrderItem,
  Schema,
  buildModels,
  converge,
  dropSchema,
  generateHexToken,
  hashPassword,
  migrate,
  syncSchema,
  tableBuilder,
  truncate,
  verifyPassword
});
