var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/utils.ts
var utils_exports = {};
__export(utils_exports, {
  generateHexToken: () => generateHexToken,
  hashPassword: () => hashPassword,
  verifyPassword: () => verifyPassword
});
module.exports = __toCommonJS(utils_exports);
var import_crypto = require("crypto");
var import_util = require("util");
var import_assert = require("assert");
function generateHexToken(size) {
  return (0, import_crypto.randomBytes)(size).toString("hex");
}
async function parsePassword(password) {
  const salt = (await (0, import_util.promisify)(import_crypto.randomBytes)(16)).toString("hex");
  const iterations = 1e5;
  const keylen = 64;
  const digest = "sha256";
  const derivedKey = await (0, import_util.promisify)(import_crypto.pbkdf2)(password, salt, iterations, keylen, digest);
  return `pbkdf2_${digest}$${iterations}$${salt}$${derivedKey.toString("hex")}`;
}
async function hashPassword(plainTextPassword, options = {}) {
  if (plainTextPassword.trim().length < 6) {
    throw new Error("Password invalid: too short");
  }
  if (options.legacy) {
    return parsePassword(plainTextPassword);
  }
  const saltBuffer = await (0, import_util.promisify)(import_crypto.randomBytes)(16);
  const iterations = 15e4;
  const keylen = 32;
  const digest = "sha256";
  const derivedKeyBuffer = await (0, import_util.promisify)(import_crypto.pbkdf2)(plainTextPassword, saltBuffer, iterations, keylen, digest);
  const salt = saltBuffer.toString("base64");
  const derivedKey = derivedKeyBuffer.toString("base64");
  return `pbkdf2_${digest}$${iterations}$${salt}$${derivedKey}`;
}
async function verifyPassword(plainTextPassword, passwordHash, options = {}) {
  const legacy = options.legacy || false;
  const [algorithm, iterations, salt, derivedKey] = passwordHash.split("$");
  (0, import_assert.strictEqual)(algorithm, "pbkdf2_sha256", "Invalid algorithm.");
  (0, import_assert.strictEqual)(typeof iterations, "string", "Invalid password format.");
  (0, import_assert.strictEqual)(typeof salt, "string", "Invalid password format.");
  (0, import_assert.strictEqual)(typeof derivedKey, "string", "Invalid password format.");
  (0, import_assert.strictEqual)(isNaN(parseInt(iterations, 10)), false, "Invalid password format.");
  const saltBuffer = Buffer.from(salt, legacy ? "hex" : "base64");
  const derivedKeyBuffer = Buffer.from(derivedKey, legacy ? "hex" : "base64");
  const digest = "sha256";
  const password = await (0, import_util.promisify)(import_crypto.pbkdf2)(plainTextPassword, legacy ? saltBuffer.toString("hex") : saltBuffer, parseInt(iterations, 10), derivedKeyBuffer.length, digest);
  return (0, import_crypto.timingSafeEqual)(password, derivedKeyBuffer);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  generateHexToken,
  hashPassword,
  verifyPassword
});
